#include <graph.h>
#include <string.h>
#include <user.h>

#include "chinese.hpp"
#include "file.hpp"
#include "except.hpp"

static const BYTE mask[]={0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};

Chinese::Chinese(char *szLibname, int size)
{
    m_szLib=NULL;
    if(szLibname==NULL)
        return ;
    if(size!=16)
        return ;
    m_szLib= new char[strlen(szLibname)];
    if(!m_szLib) throw XMemoryException();
    strcpy(m_szLib, szLibname);
}
Chinese::~Chinese()
{
    if(m_szLib)
        delete m_szLib;
}
void Chinese::setLib(char *szLibname, int size)
{
    if(m_szLib!=NULL)
        delete m_szLib;
    m_szLib=NULL;
    if(szLibname==NULL)
        return ;
    if(size!=16)
        return ;
    m_szLib= new char[strlen(szLibname)];
    if(!m_szLib) throw XMemoryException();
    strcpy(m_szLib, szLibname);
}

long Chinese::code2Off(unsigned short code)
{
	long q,w;

	q=((code>>8)&0xff)-0xa1;
	w=(code&0xff)-0xa1;

	return (q*94L+w)<<5;
}

long Chinese::code2Index(unsigned short code)
{
	long q,w;

	q=((code>>8)&0xff)-0xa1;
	w=(code&0xff)-0xa1;

    return (q*94L+w);
}

void Chinese::printString(int x, int y, char *chin, short color)
{
	short xx,yy,ccc,code, n,oldcolor;
    CFile file;
	long len;
	byte pDot[32];

    file.open(m_szLib);

	oldcolor=_getcolor();

	_setcolor(color);
    for(n=0;chin[n]!=0;)
	{
		if((byte)(chin[n]) > 0xa0)
		{
			code=((chin[n]<<8)&0xff00)+(chin[n+1]&0xff);
            len=code2Off(code);
            file.seek(len);
            file.read(pDot, 32);
			for(yy=0;yy<16;yy++)
				for(xx=0;xx<2;xx++)
					for(ccc=0;ccc<8;ccc++)
						if((pDot[(yy<<1)+xx]&mask[ccc])!=0)
							_setpixel(x+(xx<<3)+ccc,y+yy);
			x+=16;
			n+=2;
		}
		else
		{
            len=file.getLength();
            len-=4096;
            len+=chin[n]*16;
            file.seek(len);
            file.read(pDot, 16);
	
            for(yy=0;yy<16;yy++)
                for(xx=0;xx<8;xx++)
                    if((pDot[yy]&mask[xx])!=0)
                        _setpixel(x+xx,y+yy+2);
			x+=8;
			n++;
		}
	}
	_setcolor(oldcolor);
    file.close();
}

