#if !defined(I8250_H)
#define I8250_H

#include <user.h>

#define REG_THR     0    /* TX buffer */
#define REG_RBR     0    /* RX buffer */
#define REG_LSB     0    /* Divisor latch LO byte */
#define REG_MSB     1    /* Divisor latch HI byte */
#define REG_IER     1    /* Interrupt enable register */
#define REG_IIR     2    /* Interrupt identification register */
#define REG_LCR     3    /* Line control register */
#define REG_LSR     5    /* Line Status register */
#define REG_MCR     4    /* Modem control register */
#define REG_MSR     6    /* Modem status register */

////////////////////////  a clear explanation of i8250 registers ////////
/*
LCR:
	x   x   x   x   x   x   x   x   x

*/////////////////////////////////////////////////////////////////////////

class XSIOException: public XException
{
}
class XSIOTimeOutException: public XSIOException
{
}
class XSIOHardwareException: public XSIOException
{
}

class CCom
{
	private:
        static int m_nPortBase;
        static BOOL m_bUseInt;
        static INTERRUPT m_vector;
    public:
        enum PARITY {NOPARITY, EVENPARITY, ODDPARITY};
    protected:
        int getAddr(int off) {return m_nPortBase+off;}
        int getBaudLo(long baudrate) {return (184320L/baudrate) & 0xff; }
        int getBaudHi(long baudrate) {return ((184320L/baudrate)>>8) & 0xff; }
        static interrupt m_isr();
#if defined(__386__)
        int getPortBase(int nPort)
        {
            if(nPort==2)
                return (short)(*((short far *)0x00402L);
            else
                return (short)(*((short far *)0x00400L);
        }
#else
        int getPortBase(int nPort)
        {
            if(nPort==2)
                return (short)(*((short far *)0x00400002L);
            else
                return (short)(*((short far *)0x00400000L);
        }
#endif
	public:
        CCom();
        ~CCom();
        void open(short nport=COM1, long baudrate=9600, short charbits=8,
            short stopbits=1, PARITY parity=NOPARITY, BOOL bUseInt);
        void close();

        void write(char *, int size); // send one char
        void writeChar(char); // send a group of data
        void writeInt(short);
        void writeLong(long);
        void writeString(short,...);

        void read(char *, int size); // send one char
        char readChar(); // send a group of data
        short readInt();
        long readLong();

        void setParityBit();
        void clearParityBit();
} ;

#endif
