#if !defined(GUI_CONTROL_HPP_)
#define GUI_CONTROL_HPP_

#include "except.hpp"
#include <user.h>

class CPoint
{
    public:
        int x,y;
    public:
        CPoint();
        CPoint(int, int);
        CPoint(CPoint &);
        ~CPoint();
        void setPoint(int, int);
        void setPoint(CPoint &);
};

class CRect
{
    public:
        int x,y,cx,cy;
    public:
        CRect();
        CRect(int, int , int, int);
        CRect(CRect &);
        ~CRect();
        void setRect(int, int, int, int);
        void setRect(CRect &);
        BOOL isPtInside(CPoint &);
        BOOL isPtInside(int, int);
} ;

// HORIZONTAL and VERTICAL s are for bumps only

class CShape
{
    public:
        enum BOXSHAPE {PLAIN=0, CONCAVE=1, CONVEX=2, HORIZONTAL=4,
            VERTICAL=8};
    public:
        void drawBox(int, int, int, int, BOXSHAPE, BOOL bPaintBkGrnd=FALSE);
        void drawGroup(int, int, int, int, BOXSHAPE);
        void drawBump(int x, int y, int len, int);
};

class CButton
{
    private:
        char *m_pText;
        BOOL m_bEnable;
        CRect m_rcBtn;
        COLORS m_color;
    public:
        enum BUTTONSTATE {PRESSED, RELEASED};
    public:
        CButton(char *, COLORS, int, int, int, int, BOOL bEn=TRUE);
        CButton(char *, COLORS, CRect &, BOOL bEn=TRUE);
        CButton(CButton &);
        ~CButton();
        void click();
        void enable(BOOL bEn=TRUE);
        void disable() {enable(FALSE);}
        void paint(BUTTONSTATE bs=RELEASED);
        void setFocus(BOOL bSet=FALSE);
        BOOL isEnable() {return m_bEnable;}
        BOOL isPtInside(CPoint &pt) {return m_rcBtn.isPtInside(pt);}
        BOOL isPtInside(int x, int y) {return m_rcBtn.isPtInside(x,y);}
} ;

class CStatic
{
	private:
		char *text;
		short x,y,len;
		short color,bkcolor;
        CShape::BOXSHAPE shape;
        BOOL m_bShow;
	public:
		CStatic(short x, short y, char const *text, short len,
            CShape::BOXSHAPE shape=CShape::PLAIN, COLORS color=BLACK, COLORS bkcolor=LIGHTGRAY);
		CStatic(CStatic const &);
		~CStatic();
        void show();
        void hide();
        void setText(char const *,short n=-1);
        void setText(int n, int radix=10);
        void setColor(COLORS color=BLACK, COLORS bkcolor=LIGHTGRAY);
        void setShape(CShape::BOXSHAPE);
        void move(short, short);
        void clear();
        BOOL isBlank();
} ;

class CEdit
{
	private:
        char *m_pText;
        short m_nX,m_nY,m_nLen;
        short m_nColor,m_nBkcolor;
        CShape::BOXSHAPE m_bs;
        BOOL m_bShow;
	public:
		CEdit(short x, short y, char const *text, short len,
            CShape::BOXSHAPE shape=CShape::CONCAVE, COLORS color=WHITE, COLORS bkcolor=BLACK);
		CEdit(CEdit const &);
		~CEdit();

        void clear();
        short getText(char *);
        short input(char *);
        BOOL isBlank();
        void move(short, short);
        void setBkColor(COLORS bkcolor);
        void setColor(COLORS clr=WHITE, COLORS bkcolor=-1);
        void setText(char const *,short n=-1);
        void setText(int n, int radix=10);
        void setVisible(BOOL bVis);
        void show();
    protected:
        BOOL usrIsChin(short key);
        void clearChar(short x, short y, COLORS color);
        BOOL usrIsPrint(short);
} ;

#endif
