#include <stdio.h>
#include <user.h>

#include "file.hpp"

CFile::CFile()
{
    m_hFile=NULL;
}
CFile::~CFile()
{
    close();
}

void CFile::close()
{
    if(m_hFile)
    {
        ::fclose(m_hFile);
        m_hFile=NULL;
    }
}
void CFile::flush()
{
    if(::fflush(m_hFile))
        throw XFileIOException();
}
long CFile::getLength()
{
	long len,lCurrent;

    lCurrent=tell();
    seek(0L,SEEK_END);
    len=tell();
    seek(lCurrent,SEEK_SET);
	return len;
}
BOOL CFile::isExist(char *szFilename)
{
	BOOL bExist=FALSE;

    m_hFile=::fopen(szFilename, "rb");
    if(m_hFile)
	{
		bExist=TRUE;
        ::fclose(m_hFile);
        m_hFile=NULL;
	}
	return bExist;
}
void CFile::open(char *szFilename, char *mode)
{
    if(mode==NULL)
        m_hFile=::fopen(szFilename, "rb");
    else
        m_hFile=::fopen(szFilename, mode);
    if(!m_hFile)
        throw XFileIOException();
}
void CFile::read(void *ptr, size_t size)
{
    ::fread(ptr, sizeof(char), size, m_hFile);
    if(ferror(m_hFile))
        throw XFileIOException();
}
int  CFile::readChar()
{
    int n=::fgetc(m_hFile)&0xff;
    if(ferror(m_hFile))
        throw XFileIOException();
    return n;
}
int  CFile::readInt()
{
    int n;
    n=::fgetc(m_hFile)&0xff;
    n|=(::fgetc(m_hFile)<<8)&0xff00;
    if(ferror(m_hFile))
        throw XFileIOException();
    return n;
}
long CFile::readLong()
{
    long n;
    n=readInt()&0xffffL;
    n|=((long)readInt()<<16)&0xffff0000L;
    if(ferror(m_hFile))
        throw XFileIOException();
    return n;
}
void CFile::readString(char *str, short size)
{
    ::fgets(str, size, m_hFile);
    if(ferror(m_hFile))
        throw XFileIOException();
}
void CFile::seek(long offset, int pos)
{
    if(::fseek(m_hFile, offset, pos))
        throw XFileIOException();
}
void CFile::setPos(fpos_t *poffset)
{
    if(::fsetpos(m_hFile, poffset))
        throw XFileIOException();
}
long CFile::tell()
{
    long lLen;
    lLen=::ftell(m_hFile);
    if(lLen==-1L)
        throw XFileIOException();
    return lLen;
}
void CFile::tmpfile()
{
    m_hFile=::tmpfile();
    if(m_hFile==NULL)
        throw XFileIOException();
}
void CFile::write(void *ptr, size_t size)
{
    size_t sz=::fwrite(ptr, sizeof(char), size, m_hFile);
    if(sz<size)
        throw XFileIOException();
}
void CFile::writeChar(int ch)
{
    ::fputc(ch, m_hFile);
}
void CFile::writeInt(int n)
{
    ::fputc(n&0xff, m_hFile);
    ::fputc((n>>8)&0xff, m_hFile);
}
void CFile::writeLong(long n)
{
    writeInt(n&0xffffL);
    writeInt((n>>16)&0xffffL);
}
void CFile::writeString(char *str)
{
    ::fputs(str, m_hFile);
}
