#if !defined(_GYMCLASS_HPP)
#define _GYMCLASS_HPP

#include <graph.h>
#include <stdio.h>
#include <user.h>

#include "except.hpp"

#define BLACK   0          /* dark colors */
#define BLUE    1
#define GREEN   2
#define CYAN    3
#define RED     4
#define MAGENTA 5
#define BROWN   6
#define LIGHTGRAY   7
#define DARKGRAY    8           /* light colors */
#define LIGHTBLUE   9
#define LIGHTGREEN  10
#define LIGHTCYAN   11
#define LIGHTRED    12
#define LIGHTMAGENTA 13
#define YELLOW      14
#define WHITE       15

#define MAXROUND       15


#define szChineseLib    "FONT.LIB"
#define TOTAL_MENU		6

typedef short   COLORS;

typedef struct tagMenuFrame
{
    short x,y,sx,sy;
    char *savebuf;
    char **menutext;
    char *hotkey;
    short count;
    BOOL active;
} MenuFrame;

typedef struct
{
    unsigned short id; // 'TR' 0x5254
    short width;
    short height;
    unsigned short color;
    unsigned short version;
	long offset_r;
	long len_r;
	long offset_g;
	long len_g;
	long offset_b;
	long len_b;
	unsigned char reserved[14];
} BICHEAD, *NPBICHEAD; // 48 bytes structure


typedef enum tagBOXSHAPE{PLAIN,CONCAVE,CONVEX} BOXSHAPE;
typedef enum {BTN_PRESSED, BTN_RELEASED} BUTTONSTATE;

typedef struct tagRGBQUAD
{
    byte    rgbBlue;
    byte    rgbGreen;
    byte    rgbRed;
    byte    rgbReserved;
} RGBQUAD;

typedef struct tagBITMAPINFOHEADER
{
    short    bfType;
    long   bfSize;
    short    bfReserved1;
    short    bfReserved2;
    long   bfOffBits;

    long   biSize;
    long    biWidth;
    long    biHeight;
    short    biPlanes;
    short    biBitCount;
    long   biCompression;
    long   biSizeImage;
    long    biXPelsPerMeter;
    long    biYPelsPerMeter;
    long   biClrUsed;
    long   biClrImportant;
    RGBQUAD bmiColor[2];
} BITMAP_HEADER;

typedef struct tagMsg
{
    int code;
    int x;
    int y;
} MSG;


class CPoint
{
    public:
        int x,y;
    public:
        CPoint();
        CPoint(int, int);
        CPoint(CPoint &);
        ~CPoint();
        void setPoint(int, int);
        void setPoint(CPoint &);
};

class CRect
{
    public:
        int x,y,cx,cy;
    public:
        CRect();
        CRect(int, int , int, int);
        CRect(CRect &);
        ~CRect();
        void setRect(int, int, int, int);
        void setRect(CRect &);
        BOOL isPtInside(CPoint &);
        BOOL isPtInside(int, int);
} ;

class CButton
{
    private:
        char *m_pText;
        BOOL m_bEnable;
        CRect m_rcBtn;
        COLORS m_color;
    public:
        CButton(char *, COLORS, int, int, int, int, BOOL bEn=TRUE);
        CButton(char *, COLORS, CRect &, BOOL bEn=TRUE);
        CButton(CButton &);
        ~CButton();
        void click();
        void enable(BOOL bEn=TRUE);
        void disable() {enable(FALSE);}
        void paint(BUTTONSTATE bs=BTN_RELEASED);
        void setFocus(BOOL bSet=FALSE);
        BOOL isEnable() {return m_bEnable;}
        BOOL isPtInside(CPoint &pt) {return m_rcBtn.isPtInside(pt);}
        BOOL isPtInside(int x, int y) {return m_rcBtn.isPtInside(x,y);}
} ;

class CImage
{
    private:
        char *m_ptr;
        unsigned m_size;
    public:
        CImage();
        ~CImage();
        BOOL create(int x, int y, char *ptr, COLORS color=WHITE);
        BOOL dup(CImage &);
        BOOL loadFromFile(char *szFilename, COLORS color=WHITE);
        void destroy();

        BOOL getImage(int x, int y, int cx, int cy);
        BOOL getImage(CRect &rect);
        void putImage(int x, int y, short mode=_GPSET);

        void invert(void);
        void dump(char *szFilename); // for debug purpose
};

class CKeybrd
{
    private:
        short m_nUngetKey;
        short m_nKey;
    public:
        CKeybrd();
        short getKey();
        void ungetKey();
    protected:
        short scanKeybrd();
} ;

class CFile
{
    private:
        FILE *m_hFile;
    public:
        CFile();
        // CFile(char *, char *md=NULL);
        ~CFile();

        void close();
        void flush();
        long getLength();
        BOOL isExist(char *);
        void open(char *, char *md=NULL);
        void read(void *ptr, size_t size);
        int readChar();
        int readInt();
        long readLong();
        void readString(char *, short size);
        void seek(long offset, int pos=SEEK_SET);
        void setPos(fpos_t *poffset);
        long tell();
        void tmpfile();
        void write(void *ptr, size_t size);
        void writeChar(int);
        void writeInt(int);
        void writeLong(long);
        void writeString(char *);

        FILE *getFile() const {return m_hFile;}
} ;

typedef struct tagFontHead
{
    short id; // 'FT'
    short version; // 0x0100
} FONT_HEAD;

typedef struct tagFontChunck
{
    short id;   // user defined font id
    short type; // 0x01 mono color dots, 0xff, terminated chunk
    short num;
    short width;
    short height;
    char comment[8];
} FONT_CHUNCK;

// 1: original bitmap font, 2: font 1.0 file, 3: Mono Bitmap
// 4: 16 color bitmap
class CFont
{
    private:
        CFile m_file;
        CImage m_imgFont;
        char *m_ptr;
        int m_nNum;
        long m_lOffset;
        int width;
        int height;
        int type;
    public:
        CFont();
        ~CFont();

        void setDim(int x, int y);
        void getDim(CPoint *pt) const;
        void getDim(int *px, int *py) const;
        int getType() const {return type;}
        int loadFont(char *szFilename, short fontid=-1);
        int loadFontIntoMem(char *szFilename, short fontid=-1);
        void writeAt(int x, int y, long index, COLORS color=WHITE,
            BOOL bTransparent=FALSE);
} ;

class CStatic
{
	private:
		char *text;
		short x,y,len;
		short color,bkcolor;
		BOXSHAPE shape;
        BOOL m_bShow;
	public:
		CStatic(short x, short y, char const *text, short len,
            BOXSHAPE shape=PLAIN, COLORS color=BLACK, COLORS bkcolor=LIGHTGRAY);
		CStatic(CStatic const &);
		~CStatic();
        void show();
        void hide();
        void setText(char const *,short n=-1);
        void setText(int n, int radix=10);
        void setColor(COLORS color=BLACK, COLORS bkcolor=LIGHTGRAY);
        void setShape(BOXSHAPE);
        void move(short, short);
        void clear();
        BOOL isBlank();
} ;

class CEdit
{
	private:
        char *m_pText;
        short m_nX,m_nY,m_nLen;
        short m_nColor,m_nBkcolor;
        BOXSHAPE m_bs;
        BOOL m_bShow;
	public:
		CEdit(short x, short y, char const *text, short len,
            BOXSHAPE shape=CONCAVE, COLORS color=WHITE, COLORS bkcolor=BLACK);
		CEdit(CEdit const &);
		~CEdit();

        void clear();
        short getText(char *);
        short input(char *);
        BOOL isBlank();
        void move(short, short);
        void setBkColor(COLORS bkcolor);
        void setColor(short COLORS=WHITE, COLORS bkcolor=-1);
        void setText(char const *,short n=-1);
        void setText(int n, int radix=10);
        void setVisible(BOOL bVis);
        void show();
    protected:
        BOOL usrIsChin(short key);
        void clearChar(short x, short y, COLORS color);
        BOOL usrIsPrint(short);
} ;


/* About m_nType member of CCounter

    m_nType:
        CNT_UNLIMTIME: Unlimited time, fixed points
        CNT_UNLIMPOINT: Limited time
*/

class CCounter
{
    public:
        enum TYPE {NIL=0, UNLIMTIME=1, UNLIMPOINT=1};

    private:
        short m_nWinRound;
        short m_nMaxRound;
        short m_nMaxPoint;
        TYPE m_nType;

        short m_nGameID;
        char *m_szGameName;

        short m_sPtA[MAXROUND];
        short m_sPtB[MAXROUND];
        // BOOL m_nsRdOver[MAXROUND];
        short m_nCurRound; // <=MAXROUND
        short m_nInService; // 0: none, 1: A, 2: B

        short m_bChangeService;
        short m_bCountRound;
        short m_nInc1, m_nInc2, m_nInc3, m_nDec1;

        BOOL m_bRoundOver;
        BOOL m_bMatchOver;

    public:
        CCounter();
        ~CCounter();

        // return 0, if OK
        // return -1 if open file error
        // return -2 if cannot allocate mem
        // other, lineno in which error occurs
        short loadScript(char *, short gameid);

        short getWinRound();
        short getMaxRound();
        short getMaxPoint();
        TYPE getType();

        short getGameID();
        void getGameName(char *);

        short getScoreA(short nRound=-1);
        short getScoreB(short nRound=-1);
        short getWinRoundA();
        short getWinRoundB();

        BOOL isRoundOver(short nRound=-1);
        BOOL isMatchOver();
        short whoWinRound(short nRound=-1);
        short whoWinMatch();

        short getInc1() const {return m_nInc1;}
        short getInc2() const {return m_nInc2;}
        short getInc3() const {return m_nInc3;}
        short getDec1() const {return m_nDec1;}

        void inc1A();
        void inc2A();
        void inc3A();
        void dec1A();

        void inc1B();
        void inc2B();
        void inc3B();
        void dec1B();

        void setService(short);
        void setRoundOver();
        void setMatchOver();

        void dump(char *); // for debug only

        short get900(short);
        short get90(short);
        short get9(short);

    protected:
} ;

const BYTE mask[]={0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};

#endif
