#include <graph.h>
#include <mem.h>
#include <user.h>

#include "except.hpp"
#include "image.hpp"


CImage::CImage()
{
    m_ptr=NULL;
    m_size=0;
}
CImage::~CImage()
{
    destroy();
}
void CImage::destroy()
{
    if(m_ptr)
        delete m_ptr;
    m_ptr=NULL;
    m_size=0;
}
BOOL CImage::create(int width, int height, char *ptr, COLORS color)
{
    destroy();
    m_size=_imagesize(0,0,width-1, height-1);
    m_ptr=new char[m_size];
    
    if(m_ptr==NULL)
    {
        m_size=0;
        throw XMemoryException();
    }
    memset(m_ptr, 0, m_size);
    *((short *)m_ptr)=width;
    *((short *)m_ptr+1)=height;
    *((short *)m_ptr+2)=0x0400;

    char *mptr=m_ptr+6;
    unsigned x, y, vx, incSrc, incDst;
    vx=(m_size-6)/height;
    incSrc=0;
    incDst=0;
    for(y=0; y<height; y++)
    {
        if(color&0x08)
            memcpy(mptr+incDst, ptr+incSrc, width/8);
        if(color&0x04)
            memcpy(mptr+incDst+(vx/4), ptr+incSrc, width/8);
        if(color&0x02)
            memcpy(mptr+incDst+(vx/2), ptr+incSrc, width/8);
        if(color&0x01)
            memcpy(mptr+incDst+(vx*3/4), ptr+incSrc, width/8);
        incSrc+=width/8;
        incDst+=vx;
    }
    return TRUE;
}
BOOL CImage::dup(CImage &image)
{
    destroy();
    if(image.m_ptr)
    {
        m_ptr=new char[image.m_size];
        if(!m_ptr)
            throw XMemoryException();
        memcpy(m_ptr, image.m_ptr, m_size);
    }
    else
        m_ptr=NULL;
    m_size=image.m_size;
    return TRUE;
}
BOOL CImage::loadFromFile(char *szFilename, COLORS color)
{
    CFile file;
    char *ptr, *p;
    int n, vx;

    try
    {
        file.open(szFilename);
    }
    catch(XFileIOException e)
    {
        return FALSE;
    }

    BICHEAD bichead;
    BITMAP_HEADER bhead;
    memset(&bichead, 0, sizeof(BICHEAD));
    file.read(&bichead, sizeof(BICHEAD));
    if(bichead.id==0x04d42) // bitmap file
    {
        file.seek(0L);
        long len=file.getLength()-sizeof(BITMAP_HEADER);
        file.read(&bhead, sizeof(BITMAP_HEADER));
        ptr=new char[len];
        if(!ptr) throw XMemoryException();
        // file.seek(sizeof(BITMAP_HEADER));
        file.read(ptr, len);
        file.close();
        p=new char[len];
        if(!p) {delete ptr; throw XMemoryException();}
        vx=len/bhead.biHeight;
        for(n=0; n<bhead.biHeight; n++)
            memcpy(p+vx*(bhead.biHeight-n-1), ptr+vx*n, vx);
        delete ptr;
        create(vx*8, bhead.biHeight, p, color);
        delete p;
        return TRUE;
    }
    if((bichead.id==0x5254)&&(bichead.len_r!=0))
    {
        ptr=new char[bichead.len_r];
        if(ptr==NULL) throw XMemoryException();
        file.seek(bichead.offset_r);
        file.read(ptr, bichead.len_r);
        file.close();
        create(bichead.width, bichead.height, ptr, color);
        delete ptr;
        return TRUE;
    }
    return FALSE;
}
BOOL CImage::getImage(int x, int y, int cx, int cy)
{
    destroy();
    m_size=_imagesize(x, y, cx, cy);
    m_ptr=new char[m_size];
    if(m_ptr==NULL)
    {
        m_size=0;
        throw XMemoryException();
    }
    _getimage(x, y, cx, cy, m_ptr);
    return TRUE;
}
BOOL CImage::getImage(CRect &rect)
{
    destroy();
    m_size=_imagesize(rect.x, rect.y, rect.cx, rect.cy);
    m_ptr=new char[m_size];
    if(m_ptr==NULL)
    {
        m_size=0;
        throw XMemoryException();
    }
    _getimage(rect.x, rect.y, rect.cx, rect.cy, m_ptr);
    return TRUE;
}
void CImage::putImage(int x, int y, short mode)
{
    if(m_ptr)
        _putimage(x, y, m_ptr, mode);
}
void CImage::invert()
{
    char *ptr=m_ptr;
    for(unsigned n=6; n<m_size; n++)
        ptr[n]=~ptr[n];
}
void CImage::dump(char *szFileName)
{
    CFile debug;
    try
    {
        debug.open(szFileName, "wb");
        debug.write(&m_size,sizeof(unsigned));
        debug.write(&m_ptr,m_size);
    }
    catch(XFileIOException e)
    {
    }
    debug.close();
}
