#include <dos.h>
#include <assert.h>

#include "misc.hpp"
#include "mouse.hpp"

CMouse::CMouse()
{
	button=bcount=x=y=0;
    bLInvert=FALSE;
    bMInvert=FALSE;
    bRInvert=FALSE;
}
CMouse::~CMouse()
{
}

// return a boolean value, indicate wether or not mouse can be used
// bcount is number of MOUSE KEYS
short CMouse::reset()
{
	REGS r;
	r.w.ax=0;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
	bcount=r.w.bx;
	return r.w.ax;
}

void CMouse::show()
{
	REGS r;
	r.w.ax=0x01;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
}
void CMouse::hide()
{
	REGS r;
	r.w.ax=0x02;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
}
void CMouse::getPos()
{
	REGS r;
	r.w.ax=0x03;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
	button=r.w.bx;
	x=r.w.cx;
	y=r.w.dx;
}
void CMouse::setPos(short mx, short my)
{
	REGS r;
	r.w.ax=0x04;
	r.w.cx=mx;
	r.w.dx=my;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
}
void CMouse::setPos(CPoint &pt)
{
	REGS r;
	r.w.ax=0x04;
	r.w.cx=pt.x;
	r.w.dx=pt.y;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
}
void CMouse::getPress(MOUSEKEY mk)
{
	REGS r;
	r.w.ax=0x05;
	r.w.bx=mk;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
	button=r.w.ax;
	bcount=r.w.bx;
	x=r.w.cx;
	y=r.w.dx;
}
void CMouse::getRelease()
{
	REGS r;
	r.w.ax=0x06;
	r.w.bx=0x07;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
	button=r.w.ax;
	bcount=r.w.bx;
	x=r.w.cx;
	y=r.w.dx;
}
void CMouse::setHLimit(short minx,short maxx)
{
	REGS r;
	r.w.ax=0x07;
	r.w.cx=minx;
	r.w.dx=maxx;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
}
void CMouse::setVLimit(short miny,short maxy)
{
	REGS r;
	r.w.ax=0x08;
	r.w.cx=miny;
	r.w.dx=maxy;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
}
void CMouse::setHLimit(CPoint &pt)
{
	REGS r;
	r.w.ax=0x07;
	r.w.cx=pt.x;
	r.w.dx=pt.x;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
}
void CMouse::setVLimit(CPoint &pt)
{
	REGS r;
	r.w.ax=0x08;
	r.w.cx=pt.y;
	r.w.dx=pt.y;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
}
void CMouse::setBound(short minx,short miny,short maxx,short maxy)
{
    setHLimit(minx,maxx);
    setVLimit(miny,maxy);
}
void CMouse::setBound(CRect &rc)
{
    setHLimit(rc.x,rc.cx);
    setVLimit(rc.y,rc.cy);
}
void CMouse::setGShape(const unsigned short *pmask,short hotx,short hoty)
{
	REGS r;
	SREGS sr;
	unsigned short far *ptr;
	r.w.ax=0x09;
	r.w.bx=hotx;
	r.w.cx=hoty;
    ptr=(unsigned short far *)pmask;

#if defined(__386__)
    r.x.edx=FP_OFF(ptr);
	int386(0x33, &r, &r);
#else
    sr.es=FP_SEG(ptr);
    r.w.dx=FP_OFF(ptr);
    int86x(0x33,&r,&r,&sr);
#endif
}
void CMouse::setTShape(short select,short begin,short end)
{
	REGS r;
	r.w.ax=0x0a;
	r.w.bx=select;
	r.w.cx=begin;
	r.w.dx=end;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
}
void CMouse::getRel()
{
	REGS r;
	r.w.ax=0x0b;
#if defined(__386__)
	int386(0x33, &r, &r);
#else
    int86(0x33,&r,&r);
#endif
	x=r.w.cx;
	y=r.w.dx;
}
BOOL CMouse::isLBtnDown()
{
    return button&(int)MK_LEFT?TRUE:FALSE;
}
BOOL CMouse::isMBtnDown()
{
    return button&(int)MK_MID?TRUE:FALSE;
}
BOOL CMouse::isRBtnDown()
{
    return button&(int)MK_RIGHT?TRUE:FALSE;
}
BOOL CMouse::isBothBtnDown()
{
    return button&(int)MK_BOTH?TRUE:FALSE;
}
BOOL CMouse::isAllBtnDown()
{
    return button&(int)MK_ALL?TRUE:FALSE;
}

BOOL CMouse::isLBtnUp()
{
    return button&(int)MK_LEFT?FALSE:TRUE;
}
BOOL CMouse::isMBtnUp()
{
    return button&(int)MK_MID?FALSE:TRUE;
}
BOOL CMouse::isRBtnUp()
{
    return button&(int)MK_RIGHT?FALSE:TRUE;
}
BOOL CMouse::isBothBtnUp()
{
    return button&(int)MK_BOTH?FALSE:TRUE;
}
BOOL CMouse::isAllBtnUp()
{
    return button&(int)MK_ALL?FALSE:TRUE;
}

