#include <string.h>
#include <stdlib.h>
#include <mem.h>
#include <ctype.h>

#include "except.hpp"
#include "token.hpp"

static const char *szTokenName[]={
    "NIL", "VAR", "STRING", "NUMBER", "SYMBOL", "EQUAL", "IDNUMBER",
    "EOL", "TEOF", "SPACE", "ERROR", "COMMENT"
    };

CToken::CToken()
{
    m_pStart=m_pCur=NULL;
    m_nLen=0;
    m_nLast=0;
    m_eType=NIL;
    m_szName[0]='\0';
    m_lValue=0L;
    m_bDuplicated=FALSE;
    m_nLineno=0;
}
CToken::~CToken()
{
    if(m_bDuplicated)
        delete m_pStart;
}

void CToken::unget()
{
    m_pCur-=m_nLast;
}
BOOL CToken::isSymbol(char ch)
{
    if(strchr(",!()%$@<>.[]", ch)==NULL)
        return FALSE;
    return TRUE;
}
BOOL CToken::isVarFirst(char ch)
{
    if(isalpha(ch))
        return TRUE;
    if(ch=='_') return TRUE;
    if(byte(ch)>=0xa0)
        return TRUE;
    return FALSE;
}
BOOL CToken::isVarNext(char ch)
{
    if(isalnum(ch))
        return TRUE;
    if(ch=='_') return TRUE;
    if(byte(ch)>=0xa0)
        return TRUE;
    return FALSE;
}

BOOL CToken::init(char *p, unsigned len, BOOL bDup, short nCase)
{
    if(len==0)
        len=strlen(p);
    if(bDup)
    {
        m_pStart=new char[len+1];
        if(m_pStart==NULL)
            throw XMemoryException();
        memcpy(m_pStart, p, len);
    }
    else
        m_pStart=p;
    m_pCur=m_pStart;
    if(nCase==CASE_INSENSITIVE)
        strupr(m_pStart);
    m_bDuplicated=bDup;
    m_nLen=len;
    m_nLineno=0;
    return TRUE;
}
void CToken::getNext()
{
    char *ptr, sztmp[MAXSTRING];
    ptr=m_pCur;

    m_lValue=0;
    m_szName[0]=0;

    if( (m_pCur-m_pStart)>m_nLen )
	{
        m_eType=CToken::TEOF;
        m_nLast=0;
	}
    else if( (*ptr=='\0') || (*ptr==0x1a) )
	{
        m_eType=CToken::TEOF;
        m_nLast=0;
	}
    else if( (*ptr=='/')&&(*(ptr+1)=='/') )
	{
        ptr++;
        for(;;)
        {
            ptr++;
            if( (*ptr=='\0') || (*ptr=='0x1a') || ((ptr-m_pStart)>m_nLen) )
                break;
            if(*ptr==0x0d)
            {
                if(*(ptr+1)==0x0a)
                    ptr++;
                break;
            }
        }
        m_eType=CToken::COMMENT;
        m_nLast=ptr-m_pCur;
	}
    else if( (*ptr=='/')&&(*(ptr+1)=='*') )
    {
        ptr++;
        for(;;)
        {
            ptr++;
            if( (*ptr=='\0') || (*ptr=='0x1a') || ((ptr-m_pStart)>m_nLen) )
            {
                m_eType=CToken::ERROR;
                m_nLast=ptr-m_pCur;
                return ;
            }
            if(*ptr==0x0d)
                m_nLineno++;
            if((*ptr=='*')&&(*(ptr+1)=='/'))
            {
                ptr+=2;
                break;
            }
        }
        m_eType=CToken::COMMENT;
        m_nLast=ptr-m_pCur;
    }
	else if( isspace(*ptr) )
	{
		if(*ptr==0x0d)
        {
            m_eType=CToken::EOL;
            if(*(ptr+1)==0x0a)
            {
                ptr+=2;
                m_nLast=2;
            }
            else
            {ptr++; m_nLast=1;}
            m_nLineno++;
        }
        else
        {
            m_eType=CToken::SPACE;
            ptr++;
            m_nLast=1;
        }
	}
    else if( *ptr=='#' )
    {
        ptr++;
        m_nLast=1;
        m_eType=IDNUMBER;
    }
    else if( *ptr=='=' )
    {
        ptr++;
        m_nLast=1;
        m_eType=EQUAL;
    }
    else if( isSymbol(*ptr) )
	{
        m_eType=CToken::SYMBOL;
        m_szName[0]=*ptr;
        m_szName[1]='\0';
		ptr++;
        m_nLast=1;
	}
	else if( isdigit(*ptr)|| (*ptr=='-') )
	{
        do
        {
            ptr++;
            if( (*ptr=='\0') || (*ptr=='0x1a') || ((ptr-m_pStart)>m_nLen) )
                break;
        } while(isdigit(*ptr));
        m_nLast=ptr-m_pCur;
        strncpy(sztmp,m_pCur,m_nLast);
        sztmp[ptr-m_pCur]='\0';
        m_eType=CToken::NUMBER;
        m_lValue=atol(sztmp);
	}
	else if(*ptr=='\"')
	{
        for(;;)
        {
            ptr++;
            if( (*ptr=='\0') || (*ptr=='0x1a') || ((ptr-m_pStart)>m_nLen) )
            {
                m_eType=CToken::ERROR;
                m_nLast=ptr-m_pCur;
                return ;
            }
            if(*ptr=='\"') break;
        }
        m_nLast=ptr-m_pCur;

        if(m_nLast>MAXSTRING)
		{
            m_eType=CToken::ERROR;
            strncpy(m_szName,m_pCur+1,MAXSTRING);
            m_szName[MAXSTRING-1]='\0';
		}
		else
		{
            m_eType=CToken::STRING;
            strncpy(m_szName,m_pCur+1,m_nLast);
            m_szName[m_nLast-1]='\0';
		}
		ptr++;
	}
    else if(isVarFirst(*ptr))
	{
        for(;;)
        {
            ptr++;
            if( (*ptr=='\0') || (*ptr=='0x1a') || ((ptr-m_pStart)>m_nLen) )
                break;
            if(!isVarNext(*ptr)) break;
        }
        m_nLast=ptr-m_pCur;
        if(m_nLast>MAXSTRING)
        {
            m_eType=CToken::ERROR;
            strncpy(m_szName,m_pCur,MAXSTRING);
            m_szName[MAXSTRING-1]='\0';
        }
		else
        {
            m_eType=CToken::VAR;
            strncpy(m_szName,m_pCur,m_nLast);
            m_szName[m_nLast]='\0';
		}
	}
	else
	{
        m_eType=CToken::ERROR;
        m_lValue=3;   // number one error;
		ptr++;
        m_nLast=1;
    }
    m_pCur=ptr;
	return ;
}

